function [sys,x0,str,ts] = sfun_v_control(t,x,u,flag,mod_type,dcpred,dcpi,dcovermod,C,max_offset)
%   Variable step S-function.
%   Simulation of one leg of the pwm-module, including non-linear
%   effects as blanking time and minimum turn-off time
switch flag,
   
  case 0,    % Initialization %
    [sys,x0,str,ts]=mdlInitializeSizes;
  
  case 3,    % Outputs %
    sys=mdlOutputs(t,x,u);

  case 4,    % GetTimeOfNextVarHit %
    sys=mdlGetTimeOfNextVarHit(t,x,u,mod_type,dcpred,dcpi,dcovermod,C,max_offset);
  case 9,    % Terminate %
    sys=mdlTerminate(t,x,u); 
    
  case {1,2}    % Unhandled flags %
    sys = []; 
    
  otherwise    % Unexpected flags %
    error(['Unhandled flag = ',num2str(flag)]);

end

% end sfun_onephase

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts]=mdlInitializeSizes

sizes = simsizes;

sizes.NumContStates  = 0;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 8;    
sizes.NumInputs      = 9;
sizes.DirFeedthrough = 1;
sizes.NumSampleTimes = 1;     % at least one sample time is needed

sys = simsizes(sizes);
clc
%
% initialize the initial conditions
%
x0  = [];
str = [];
% sets the time between two following calculation steps
						  % needed to get high quality plots of the digital signals
%                     

set_param(gcbh,'UserData',[0     ,0       ,0       ,0       ,0       ,0       ,0       ,0      ,0      ,0          ,0]);
% vector is defined by:   [t_next,u_st_a_p,u_st_a_n,u_st_b_p,u_st_b_n,u_st_c_p,u_st_c_n,periods,nonrank,I0_pre_est,Iexp]
%
%
% initialize the array of sample times
%
ts  = [-2 0];      % variable sample time

% end mdlInitializeSizes

%
%=============================================================================
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,x,u);

% the if-statement defines the s-function's digital output  
% dependent of the state of the counter vaiable k

T_tri = u(3); 
enable = u(4);
variable_vect = get_param(gcbh,'UserData');
u_st_a_p=variable_vect(2);
u_st_a_n=variable_vect(3);
u_st_b_p=variable_vect(4);
u_st_b_n=variable_vect(5);
u_st_c_p=variable_vect(6);
u_st_c_n=variable_vect(7);
sys(1)=u_st_a_p;
sys(2)=u_st_a_n;
sys(3)=u_st_b_p;
sys(4)=u_st_b_n;
sys(5)=u_st_c_p;
sys(6)=u_st_c_n;
sys(7)=T_tri;
sys(8)=enable;


% end mdlOutputs

%
%=============================================================================
% mdlGetTimeOfNextVarHit
% Return the time of the next hit for this block.  Note that the result is
% absolute time.
%=============================================================================
%(t,x,u,theta_phase,t_dead,t_min,type_signal)
function sys=mdlGetTimeOfNextVarHit(t,x,u,mod_type,dcpred,dcpi,dcovermod,C,max_offset)
variable_vect = get_param(gcbh,'UserData');

u_st = u(1);
ksi_s = u(2);
T_tri = u(3);
enable = u(4);
V1=u(5);
V2=u(6);
Ia=u(7);
Ib=u(8);
t_next=t+T_tri;
periods=variable_vect(8)+1;
nonrank=variable_vect(9);
I0pre=variable_vect(10);
I0exp=variable_vect(11);
Inp=(V2-V1)*C/T_tri;
max_offset=max_offset/2;
if dcpi==1
    kpi=u(9)*C/T_tri;
else
    kpi=0;
end
I=[Ia Ib -(Ib+Ia)]';
    
if ksi_s < 0
    ksi_s = 2*pi+ksi_s;
end

if mod_type == 1 || mod_type == 2 %Space Vector Modulation
    V_ref = u_st*3/4;
    %Correcting input
    n=3;
    if V_ref~=0
        if ksi_s < 2*pi/3
            leg=[1 2 3];
            angle=ksi_s;
            medp=[2 1];
        elseif ksi_s > 4*pi/3
            leg=[3 1 2];
            angle=ksi_s-4*pi/3;
            medp=[1 3];
        else
            leg=[2 3 1];
            angle=ksi_s-2*pi/3;
            medp=[3 2];
        end
        dec(1)=V_ref*sin(2*pi/3-angle)/(sqrt(3)/2);
        dec(2)=V_ref*sin(angle)/(sqrt(3)/2);
        if dec(1) > dec(2)
            side=[1 2];
        else
            side=[2 1];
        end
        omod=0;
        if dec(1) > dec(2) && dec(1) > 1 %Referanse utenfor sektordiagrammet
            dec(1)=1;
            if angle < pi/6
                dec(2)=0.5-sqrt(0.25-(1-V_ref^2));
                if dec(2) < 0
                    dec(2)=0;
                end
                med=3;
            else
                dec(2)=0.5+sqrt(0.25-(1-V_ref^2));
                if dec(2) > 1
                    dec(2)=0.99999;
                end
                med=4;
            end
            omod=1;
        elseif dec(2) > 1
            dec(2)=1;
            if angle > pi/2
                dec(1)=0.5-sqrt(0.25-(1-V_ref^2));
                if dec(1) < 0
                    dec(1)=0;
                end
                med=3;
            else
                dec(1)=0.5+sqrt(0.25-(1-V_ref^2));
                if dec(1) > 1
                    dec(1)=0.99999;
                end
                med=4; 
            end        
            omod=1;       
        end    
        if omod==1 && dcovermod==1 %DC-busbalansering ved overmodulasjon
            if dec(1) > dec(2)
                j=medp(1);
                m=2;
            else
                j=medp(2);           
                m=1;
            end
            if abs(I(j)) > 0.001
                des=Inp/(2*I(j));%Desired time
                if dec(m) < 0.5
                    if des > dec(m)+max_offset
                        des=dec(m)+max_offset;
                    elseif des < dec(m)-max_offset
                        des=dec(m)-max_offset;
                    end
                    dec(m)=(des>0 && des < 0.4999)*des +(des> 0.5)*0.4999;
                else
                    des=1-des;%*(-1)^m
                    if des > dec(m)+max_offset
                        des=dec(m)+max_offset;
                    elseif des < dec(m)-max_offset
                        des=dec(m)-max_offset;
                    end
                    dec(m)=(des > 0.5 && des < 0.9999)*des +(des> 0.9999)*0.9999+(des<0.5)*0.5001;            
                end
            end
        end
        for i=1:2
            j=0;
            while dec(i) > 1/(n-1)
                dec(i) = dec(i)-1/(n-1);
                j=j+1;
            end
            num(i)=j;
        end
        if num(1) == num(2)
           l=n-num(1)-1;
           r(1)=1;
           r(2)=1;
        elseif num(2) < num(1)
           l=n-num(1)-1;
           r(1)=1;
           r(2)=0;
        else
           l=n-num(2)-1;
           r(1)=0;
           r(2)=1;
        end
        if l < 1
            l=1;
        end
        nv=0;
        for i=0:l
            v0(i+1,:)=[i i i];
            v0(i+1,leg(1))=v0(i+1,leg(1))+num(1);
            v0(i+1,leg(2))=v0(i+1,leg(2))+num(2);
            nv=nv+1;
            if i<l
                vx(i+1,:) = v0(i+1,:);
                vx(i+1,leg(1))=vx(i+1,leg(1))+1;     
                vx(i+1,leg(2))=vx(i+1,leg(2))+1;
                nv=nv+1;
            end
            if dec(1) > dec(2) && i <= (l-r(1))
                vy(i+1,:) = v0(i+1,:);
                vy(i+1,leg(1))=vy(i+1,leg(1))+1;      
                nv=nv+1;
            elseif dec(2) > dec(1) && i <= (l-r(2))
                vy(i+1,:) = v0(i+1,:);
                vy(i+1,leg(2))=vy(i+1,leg(2))+1; 
                nv=nv+1;
            end
        end
        vec=[v0;vx;vy];

        if dec(1) > dec(2)
            tx=dec(2);
            ty=dec(1)-dec(2);
            t0=1/(n-1)-tx-ty;
        else
            tx=dec(1);
            ty=dec(2)-dec(1);
            t0=1/(n-1)-tx-ty;
        end
        t_bal=[ones(1,length(v0(:,1)))*t0 ones(1,length(vx(:,1)))*tx ones(1,length(vy(:,1)))*ty]';


        if omod==0 %DC-bus regulation
            dentol = 0.001;
            if nv==7
               if t==0
                   f=[0.33 0.33 0.34 0.5 0.5 0.5 0.5];
               else
               Ix=I(leg(3));
               if dec(1) > dec(2)
                  Iy=I(leg(1)); 
               else
                  Iy=I(leg(2));
               end
               if (abs(Ix*tx) > dentol || abs(Iy*ty) > dentol) && dcpred==1
                   fp=Inp/(4*(abs(Ix)*tx+abs(Iy)*ty))+0.5+kpi;
                   if fp > 1
                       fp=1;
                   elseif fp < 0
                       fp=0;
                   end
               else
                   fp=0.5+kpi;
               end
               if Ix > 0
                   fx=1-fp;
               else
                   fx=fp;
               end
               if Iy > 0
                   fy=fp;
               else
                   fy=1-fp;
               end
               f=[0.33 0.33 0.34 fx (1-fx) fy (1-fy)];
               end
            elseif nv==5
               if t==0;
                   f=[0.5 0.5 1 0.5 0.5];
               else            
                   Ix=I(leg(side(2)));
                   Iy=I(leg(3));
                   I0=I(leg(side(1)));
                   if (abs(I0*t0) > dentol || abs(Iy*ty) > dentol) && dcpred==1
                       fp=(Inp-Ix*tx*2)/(4*(abs(I0)*t0+abs(Iy)*ty))+0.5+kpi;
                       if fp > 1
                           fp=1;
                       elseif fp < 0
                           fp=0;
                       end
                   else
                       fp=0.5+kpi;
                   end               
                   if I0 > 0
                       f0=fp;
                   else
                       f0=1-fp;
                   end
                   if Iy > 0
                       fy=1-fp;
                   else
                       fy=fp;
                   end                   
               end
               f=[f0 1-f0 1 fy 1-fy];               
            else
               if t==0
                   f=[0.5 0.5 1 1];
               else            
                   if num(1)==num(2)
                       I0=I(leg(3));
                       if dec(1) > dec(2)
                           Iy=I(leg(2));
                       else
                           Iy=I(leg(1));
                       end
                       if abs(I0*t0) > dentol && dcpred==1
                            fp=(Inp-2*ty*Iy)/(4*abs(I0)*t0)+0.5+kpi;
                       else
                            fp=0.5+kpi;
                       end
                       if I0 > 0
                          f0=1-fp; 
                       else
                          f0=fp;
                       end
                   else
                       if num(1) > num(2)
                            I0=I(leg(1));
                            Ix=I(leg(2));
                       else
                            I0=I(leg(2));
                            Ix=I(leg(1));
                       end
                       if (abs(I0*t0) > dentol) && dcpred==1
                            fp=(Inp-2*tx*Ix)/(4*abs(I0)*t0)+0.5+kpi;
                       else
                            fp=0.5+kpi;
                       end
                       if I0 > 0
                           f0=fp;
                       else
                           f0=1-fp;
                       end
                  end
                  if f0 > 1
                      f0=1;
                  elseif f0 < 0 
                      f0=0;
                  end
                  f=[f0 (1-f0) 1 1];
               end
            end
        end
        if omod==1
            f =[ones(1,length(v0(:,1)))/length(v0(:,1)) ones(1,length(vx(:,1)))/length(vx(:,1)) ones(1,length(vy(:,1)))/length(vy(:,1))];    
        end    
        t_all=(t_bal.*f')';
        %%Calculating switch on half duty cycle
        d=zeros(2,3); %Vector with all the 
        for i=1:length(t_all)
            for j=1:3
               d(1,j)= d(1,j) + (vec(i,j) == 2)*t_all(i);
               d(2,j)= d(2,j) + (vec(i,j) > 0)*t_all(i);
            end
        end
    else
       d=[ones(1,3)*1/3;zeros(1,3)*2/3]; 
    end
        %Generating the controlsignals
    u_st_a_p = d(1,1)*2;
    u_st_a_n = d(2,1)*2;
    u_st_b_p = d(1,2)*2;
    u_st_b_n = d(2,2)*2;
    u_st_c_p = d(1,3)*2;    
    u_st_c_n = d(2,3)*2; 

elseif mod_type==2 || mod_type ==3 || mod_type==4


if mod_type==4 %Third harmonic injection
     u_st_a = u_st*(cos(ksi_s)-cos(3*ksi_s)/6);
     u_st_b = u_st*(cos(ksi_s+2*pi/3)-cos(3*ksi_s)/6);
     u_st_c = u_st*(cos(ksi_s+4*pi/3)-cos(3*ksi_s)/6);
elseif mod_type==3 %Sinousoidal
     u_st_a = u_st*cos(ksi_s);
     u_st_b = u_st*cos(ksi_s+2*pi/3);% "default" 
     u_st_c = u_st*cos(ksi_s+4*pi/3);
else
    u_st_a = u_st*cos(ksi_s);% "default" 
    u_st_b = u_st*cos(ksi_s-2*pi/3);
    u_st_c = u_st*cos(ksi_s-4*pi/3);
    Voff = - (max([u_st_a u_st_b u_st_c])+min([u_st_a u_st_b u_st_c]))/2;
    ua_ = mod((u_st_a + Voff + 1),1);
    ub_ = mod((u_st_b + Voff + 1),1);
    uc_ = mod((u_st_c + Voff + 1),1);
    Voff_ = 0.5-(max([ua_ ub_ uc_])+min([ua_ ub_ uc_]))/2;
    u_st_a = u_st_a + Voff + Voff_;
    u_st_b = u_st_b + Voff + Voff_;
    u_st_c = u_st_c + Voff + Voff_;   
end % end if
    u_st_a_p = (u_st_a > 0)*u_st_a;
    u_st_a_n = (u_st_a < 0)*u_st_a + 1;
    u_st_b_p = (u_st_b > 0)*u_st_b;
    u_st_b_n = (u_st_b < 0)*u_st_b + 1;
    u_st_c_p = (u_st_c > 0)*u_st_c;    
    u_st_c_n = (u_st_c < 0)*u_st_c + 1;

elseif mod_type == 5 %Double Signal

    if ksi_s >= 0 && ksi_s < pi/3
         u_st_a_p = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6);
         u_st_a_n = 0;
         u_st_b_p = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6-2*pi/3);
         u_st_b_n = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6-2*pi/3);
         u_st_c_p = 0;
         u_st_c_n = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6-2*2*pi/3);
    elseif ksi_s >= pi/3 && ksi_s <= 2*pi/3
         u_st_a_p = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6);
         u_st_a_n = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6);
         u_st_b_p = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6-2*pi/3);
         u_st_b_n = 0;
         u_st_c_p = 0;
         u_st_c_n = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6-2*2*pi/3);
    elseif ksi_s >= 2*pi/3 && ksi_s <= pi
         u_st_a_p = 0;
         u_st_a_n = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6);
         u_st_b_p = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6-2*pi/3);
         u_st_b_n = 0;
         u_st_c_p = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6-2*2*pi/3);
         u_st_c_n = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6-2*2*pi/3);
    elseif ksi_s >= pi && ksi_s <= 4*pi/3
         u_st_a_p = 0;
         u_st_a_n = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6);
         u_st_b_p = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6-2*pi/3);
         u_st_b_n = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6-2*pi/3);
         u_st_c_p = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6-2*2*pi/3);
         u_st_c_n = 0;       
    elseif ksi_s >= 4*pi/3 && ksi_s <= 5*pi/3
         u_st_a_p = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6);
         u_st_a_n = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6);
         u_st_b_p = 0;
         u_st_b_n = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6-2*pi/3);
         u_st_c_p = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6-2*2*pi/3);
         u_st_c_n = 0;

    else
         u_st_a_p = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6);
         u_st_a_n = 0;
         u_st_b_p = 0;
         u_st_b_n = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6-2*pi/3);
         u_st_c_p = (sqrt(3)/2)*u_st*cos(ksi_s-pi/6-2*2*pi/3);
         u_st_c_n = (sqrt(3)/2)*u_st*cos(ksi_s+pi/6-2*2*pi/3);

    end    
u_st_a_n=u_st_a_n+1;
u_st_b_n=u_st_b_n+1;
u_st_c_n=u_st_c_n+1;
end    
I0=0;
I0exp=0;

sys = t_next;
set_param(gcbh,'UserData',[t_next,u_st_a_p,u_st_a_n,u_st_b_p,u_st_b_n,u_st_c_p,u_st_c_n,periods,nonrank,I0,I0exp]);
% writes the variable values to the block's workspace memory

% end mdlGetTimeOfNextVarHit

%
%=============================================================================
% mdlTerminate
% Perform any end of simulation tasks.
%=============================================================================
%
function sys=mdlTerminate(t,x,u)

sys = [];
% end mdlTerminate


